@@ -1,5 +1,6 @@
+// -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: dpkgpm.cc,v 1.23 2001/05/27 04:42:19 jgg Exp $
+// $Id: dpkgpm.cc,v 1.28 2004/01/27 02:25:01 mdz Exp $
 /* ######################################################################
 
    DPKG Package Manager - Provide an interface to dpkg
@@ -143,7 +144,6 @@ bool pkgDPkgPM::RunScripts(const char *Cnf)
    
    return true;
 }
-
                                                                         /*}}}*/
 // DPkgPM::SendV2Pkgs - Send version 2 package info			/*{{{*/
 // ---------------------------------------------------------------------
@@ -314,12 +314,6 @@ bool pkgDPkgPM::RunScriptsWithPkgs(const char *Cnf)
 	 Die = !SendV2Pkgs(F);
 
       fclose(F);
-      if (Die == true)
-      {
-	 kill(Process,SIGINT);
-	 ExecWait(Process,Opts->Value.c_str(),true);
-	 return _error->Error("Failure running script %s",Opts->Value.c_str());
-      }
       
       // Clean up the sub process
       if (ExecWait(Process,Opts->Value.c_str()) == false)
@@ -328,13 +322,15 @@ bool pkgDPkgPM::RunScriptsWithPkgs(const char *Cnf)
 
    return true;
 }
-
 									/*}}}*/
 // DPkgPM::Go - Run the sequence					/*{{{*/
 // ---------------------------------------------------------------------
 /* This globs the operations and calls dpkg */
 bool pkgDPkgPM::Go()
 {
+   unsigned int MaxArgs = _config->FindI("Dpkg::MaxArgs",350);   
+   unsigned int MaxArgBytes = _config->FindI("Dpkg::MaxArgBytes",8192);
+
    if (RunScripts("DPkg::Pre-Invoke") == false)
       return false;
 
@@ -347,9 +343,9 @@ bool pkgDPkgPM::Go()
       for (; J != List.end() && J->Op == I->Op; J++);
 
       // Generate the argument list
-      const char *Args[400];
-      if (J - I > 350)
-	 J = I + 350;
+      const char *Args[MaxArgs + 50];
+      if (J - I > (signed)MaxArgs)
+	 J = I + MaxArgs;
       
       unsigned int n = 0;
       unsigned long Size = 0;
@@ -405,7 +401,7 @@ bool pkgDPkgPM::Go()
       // Write in the file or package names
       if (I->Op == Item::Install)
       {
-	 for (;I != J && Size < 1024; I++)
+	 for (;I != J && Size < MaxArgBytes; I++)
 	 {
 	    if (I->File[0] != '/')
 	       return _error->Error("Internal Error, Pathname to install is not absolute '%s'",I->File.c_str());
@@ -415,7 +411,7 @@ bool pkgDPkgPM::Go()
       }      
       else
       {
-	 for (;I != J && Size < 1024; I++)
+	 for (;I != J && Size < MaxArgBytes; I++)
 	 {
 	    Args[n++] = I->Pkg.Name();
 	    Size += strlen(Args[n-1]);
@@ -440,8 +436,8 @@ bool pkgDPkgPM::Go()
          it forks scripts. What happens is that when you hit ctrl-c it sends
 	 it to all processes in the group. Since dpkg ignores the signal 
 	 it doesn't die but we do! So we must also ignore it */
-      signal(SIGQUIT,SIG_IGN);
-      signal(SIGINT,SIG_IGN);
+      sighandler_t old_SIGQUIT = signal(SIGQUIT,SIG_IGN);
+      sighandler_t old_SIGINT = signal(SIGINT,SIG_IGN);
 		     
       // Fork dpkg
       pid_t Child = ExecFork();
@@ -452,7 +448,7 @@ bool pkgDPkgPM::Go()
 	 if (chdir(_config->FindDir("DPkg::Run-Directory","/").c_str()) != 0)
 	    _exit(100);
 	 
-	 if (_config->FindB("DPkg::FlushSTDIN",true) == true)
+	 if (_config->FindB("DPkg::FlushSTDIN",true) == true && isatty(STDIN_FILENO))
 	 {
 	    int Flags,dummy;
 	    if ((Flags = fcntl(STDIN_FILENO,F_GETFL,dummy)) < 0)
@@ -483,12 +479,16 @@ bool pkgDPkgPM::Go()
 	 if (errno == EINTR)
 	    continue;
 	 RunScripts("DPkg::Post-Invoke");
+
+         // Restore sig int/quit
+         signal(SIGQUIT,old_SIGQUIT);
+         signal(SIGINT,old_SIGINT);
 	 return _error->Errno("waitpid","Couldn't wait for subprocess");
       }
 
       // Restore sig int/quit
-      signal(SIGQUIT,SIG_DFL);
-      signal(SIGINT,SIG_DFL);
+      signal(SIGQUIT,old_SIGQUIT);
+      signal(SIGINT,old_SIGINT);
        
       // Check for an error code.
       if (WIFEXITED(Status) == 0 || WEXITSTATUS(Status) != 0)
