@@ -1,6 +1,6 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: apt-config.cc,v 1.7 2001/02/20 07:03:17 jgg Exp $
+// $Id: apt-config.cc,v 1.11 2003/01/11 07:18:44 jgg Exp $
 /* ######################################################################
    
    APT Config - Program to manipulate APT configuration files
@@ -23,10 +23,12 @@
 
 #include <config.h>
 #include <apti18n.h>
-    
+
+#include <locale.h>
 #include <iostream>
 #include <string>
 									/*}}}*/
+using namespace std;
 
 // DoShell - Handle the shell command					/*{{{*/
 // ---------------------------------------------------------------------
@@ -56,7 +58,7 @@ bool DoShell(CommandLine &CmdL)
 /* */
 bool DoDump(CommandLine &CmdL)
 {
-   _config->Dump();
+   _config->Dump(cout);
    return true;
 }
 									/*}}}*/
@@ -98,7 +100,11 @@ int main(int argc,const char *argv[])
    CommandLine::Dispatch Cmds[] = {{"shell",&DoShell},
                                    {"dump",&DoDump},
                                    {0,0}};
-   
+
+   // Set up gettext support
+   setlocale(LC_ALL,"");
+   textdomain(PACKAGE);
+
    // Parse the command line and initialize the package library
    CommandLine CmdL(Args,_config);
    if (pkgInitConfig(*_config) == false ||
